__author__ = "Matteo Spanio"
__copyright__ = "Copyright 2023, Audio Innova S.r.l."
__credits__ = ["Niccolò Pretto", "Nadir Dalla Pozza", "Sergio Canazza"]
__license__ = "GPL v3.0"
__maintainer__ = "Matteo Spanio"
__email__ = "dev2@audioinnova.com"
__status__ = "Development"
